import flask
import time
import importlib
import os
import sys
import imp
from flask import request

app = flask.Flask(__name__)

@app.route('/')
def index():
    time.sleep(5)
    return 'hello world'

@app.route('/runFunc', methods=['POST'])
def run_func():
    param = request.get_json()
    file_path = param.get('pythonFile')
    func_name = param.get('function')
    args = param.get('args')
    kwargs = param.get('kwargs')
    module = import_py_file(file_path)

    try:
        ret = getattr(module, param.get('function'))(*args, **kwargs)
    except Exception as err:
        ret = str(err)
    return flask.jsonify({'ret':ret})

def import_py_file(file_path):
    module = imp.load_source('mymod', file_path)
    return module


def main():
    app.run(threaded=True)

if __name__ == "__main__":
    main()



import time
from random import randint
import uuid


def add(m, n):
    return m + n

def sleep(n):
    time.sleep(n)
    return None

class demo:
    def __init__(self):
        print("init")
        self.a = 0

    def print_a(self):
        print(self.a)

    def get_id(self):
        time.sleep(10)
        return id(self)

    def test(self):
        self.a += 1
        return self.a
from setup_demo.Serializer import *



from setup_demo import rpc_func
import traceback
import socketserver
from setup_demo.Serializer import Serializer
import logging
from uuid import uuid4, UUID

HOST, PORT = "0.0.0.0", 8000
logging.basicConfig(format='%(levelname)s:%(asctime)s:%(message)s', level=logging.DEBUG)
uuid = lambda: str(uuid4())


def is_valid_uuid(s):
    try:
        UUID(s)
        return True
    except:
        return False


def try_except(func):
    def inner(*args, **kwargs):
        try:
            ret = func(*args, **kwargs)
            return 0, ret
        except:
            print(traceback.format_exc())
            return -1, traceback.format_exc()

    return inner


@try_except
def exec_func(func_name, args=[], kwargs={}, obj=rpc_func):
    func = getattr(obj, func_name)
    ret = func(*args, **kwargs)
    return ret


class RPCHandler(socketserver.BaseRequestHandler):
    instance_map = {}

    def handle(self) -> None:
        conn = self.request
        logging.info('client connected')
        conn.sendall('connect success'.encode())
        while True:
            data = conn.recv(2048)
            if data == b"":
                logging.info('client disconnected')
                break
            func_name, args, kwargs, instance_id = Serializer.get_func_info(data)
            if is_valid_uuid(instance_id):
                try:
                    instance = RPCHandler.instance_map[instance_id]
                except KeyError as err:
                    code, ret = -1, err.with_traceback()
                    ret_info = {"code": code, "result": ret}
                    conn.send(Serializer.serialize(ret_info))
                    break
            else:
                instance = rpc_func

            code, ret = self.exec_func(func_name, args, kwargs, instance)

            if code == 0:
                if isinstance(ret, (int, str, list, tuple, dict)):
                    ret_info = {"code": code, "result": ret}
                else:
                    instance_id = uuid()
                    RPCHandler.instance_map[instance_id] = ret
                    ret_info = {"code": code, "result": instance_id}
            else:
                ret_info = {"code": code, "reason": ret}
            conn.send(Serializer.serialize(ret_info))

    @try_except
    def exec_func(self, func_name, args=[], kwargs={}, obj=rpc_func):
        func = getattr(obj, func_name)
        ret = func(*args, **kwargs)
        return ret


def main():
    server = socketserver.ThreadingTCPServer((HOST, PORT), RPCHandler)
    logging.info('server started')
    logging.info('listening at {0}:{1}'.format(HOST, PORT))
    server.serve_forever()


if __name__ == "__main__":
    main()



import socket
from setup_demo.Serializer import Serializer
from time import time, sleep
import subprocess
from sys import executable as pyexe

time_millis = lambda :time() * 1000
HOST, PORT = "127.0.0.1", 8000


def constract_data(func_name, args=[], kwargs={}):
    return {"functionName": func_name,
            "args": args,
            "kwargs": kwargs
            }


class RPCSocketClient:

    def __init__(self, url, port):
        self.client = socket.socket()
        self.server_host = (url, port)
        self._connect_()

    def _connect_(self):
        self.client.connect(self.server_host)
        _ = self.client.recv(128)

    def __getattr__(self, item):
        if item in dir(self):
            return self.__dict__(item)

        def func(*args, **kwargs):
            return self._exec_(item, args, kwargs)

        return func

    def _func_(self, *args, **kwargs):
        return self._exec_(self.func_name, args=args, kwargs=kwargs)

    def _exec_(self, func_name, args=[], kwargs={}):
        data = constract_data(func_name, args, kwargs)
        self.client.send(Serializer.serialize(data))
        ret_info = Serializer.deserialize(self.client.recv(512))
        if ret_info['code'] == 0:
            return ret_info['result']
        else:
            raise Exception(ret_info['reason'])

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.client.send(b'')


class RPCClass(RPCSocketClient):

    def __init__(self, url, port, class_name):
        super().__init__(url, port)
        self.instance_id = None
        self.instance_init(class_name)

    def instance_init(self, class_name, args=[], kwargs={}):
        data = constract_data(class_name, args, kwargs)
        self.client.send(Serializer.serialize(data))
        ret_info = Serializer.deserialize(self.client.recv(2048))
        if ret_info.get("code") == 0:
            self.instance_id = ret_info.get("result")
        else:
            raise Exception(ret_info['reason'])

    def _exec_(self, func_name, args=[], kwargs={}):
        data = {
             "functionName": func_name,
             "args": args,
             "kwargs": kwargs,
             "instanceID": self.instance_id
        }
        self.client.send(Serializer.serialize(data))
        ret_info = Serializer.deserialize(self.client.recv(512))
        if ret_info['code'] == 0:
            return ret_info['result']
        else:
            raise Exception(ret_info['reason'])


def wait_server_started(host, port):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    while True:
        try:
            s.connect((host, port))
            print("server is started")
            break
        except Exception as err:
            print("server is not started")
        sleep(3)

    s.close()


def main():
    try:
        # p = subprocess.Popen("{0} rpc_server.py".format(pyexe))
        p = subprocess.Popen("{0} -m setup_demo.rpc_server".format(pyexe))
        wait_server_started(HOST, PORT)
        with RPCClass(HOST, PORT, "demo") as client:
            t = time_millis()
            for i in range(3):
                ret1 = client.test()
                ret2 = client.get_id()
                print(ret2)
            print(time_millis() - t)
    finally:
        p.terminate()
        ...

if __name__ == "__main__":
    main()





import os
import shutil
from datetime import datetime

from setuptools import setup, find_packages

# 移除构建的build文件夹
CUR_PATH = os.path.dirname(os.path.abspath(__file__))
path = os.path.join(CUR_PATH, 'build')
if os.path.isdir(path):
    print('INFO del dir ', path)
    shutil.rmtree(path)

setup_time = datetime.now().strftime('%Y%m%d%H%M%S')

with open("C:\\Users\\Bench\\Desktop\\1.txt", 'a') as fp:
    fp.write("setup\n")

setup(
    name='httprpc-dengzhenzhen',  # 应用名
    author='dengzhenzhen',
    version='0.0.3.' + setup_time,  # 版本号
    packages=find_packages(),  # 包括在安装包内的Python包
    include_package_data=True,  # 启用清单文件MANIFEST.in,包含数据文件
    exclude_package_data={},  # 排除文件
    install_requires=[
        'Flask',
        'requests'
    ],
    entry_points={
        'console_scripts': ['runserver = setup_demo.FlaskApp:main']
    }
)
